@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2019 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -206,7 +206,7 @@ writePacket (BrailleDisplay *brl, unsigned char type, size_t size, const unsigne
 static BraillePacketVerifierResult
 verifyPacket (
   BrailleDisplay *brl,
-  const unsigned char *bytes, size_t size,
+  unsigned char *bytes, size_t size,
   size_t *length, void *data
 ) {
   const unsigned char byte = bytes[size-1];
@@ -284,7 +284,7 @@ connectResource (BrailleDisplay *brl, const char *identifier) {
 }
 
 static int
-writeIdentityRequest (BrailleDisplay *brl) {
+writeIdentifyRequest (BrailleDisplay *brl) {
   static const unsigned char bytes[] = {CE_REQ_Identify};
   return writeBytes(brl, bytes, sizeof(bytes));
 }
@@ -305,7 +305,7 @@ brl_construct (BrailleDisplay *brl, char **parameters, const char *device) {
       unsigned char response[MAXIMUM_RESPONSE_SIZE];
 
       if (probeBrailleDisplay(brl, PROBE_RETRY_LIMIT, NULL, PROBE_INPUT_TIMEOUT,
-                              writeIdentityRequest,
+                              writeIdentifyRequest,
                               readPacket, &response, sizeof(response),
                               isIdentityResponse)) {
         if (setModel(brl, response[1])) {
