@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2022 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -88,6 +88,11 @@ extern int isFloat (float *value, const char *string);
 extern int validateFloat (float *value, const char *string, const float *minimum, const float *maximum);
 #endif /* NO_FLOAT */
 
+#if defined(HAVE_PWD_H) && defined(HAVE_GRP_H)
+extern int validateUser (uid_t *value, const char *string, gid_t *group);
+extern int validateGroup (gid_t *value, const char *string);
+#endif /* defined(HAVE_PWD_H) && defined(HAVE_GRP_H) */
+
 #define PATH_SEPARATOR_CHARACTER       '/'
 #define PARAMETER_SEPARATOR_CHARACTER  ','
 #define PARAMETER_ASSIGNMENT_CHARACTER '='
