@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2021 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -22,9 +22,15 @@
 #include "program.h"
 #include "prefs.h"
 #include "tune.h"
-#include "tune_build.h"
+#include "tune_builder.h"
 #include "message.h"
 #include "brl_dots.h"
+#include "utf8.h"
+#include "core.h"
+
+#ifdef ENABLE_SPEECH_SUPPORT
+#include "spk.h"
+#endif /* ENABLE_SPEECH_SUPPORT */
 
 typedef struct {
   unsigned char duration;
@@ -41,127 +47,127 @@ typedef struct {
 
 static const AlertEntry alertTable[] = {
   [ALERT_BRAILLE_ON] = {
-    .tune = "n64@60 n69@100"
+    .tune = "m64@60 m69@100"
   },
 
   [ALERT_BRAILLE_OFF] = {
-    .tune = "n64@60 n57@60"
+    .tune = "m64@60 m57@60"
   },
 
   [ALERT_COMMAND_DONE] = {
     .message = strtext("Done"),
-    .tune = "n74@40 r@30 n74@40 r@40 n74@140 r@20 n79@50"
+    .tune = "m74@40 r@30 m74@40 r@40 m74@140 r@20 m79@50"
   },
 
   [ALERT_COMMAND_REJECTED] = {
     .tactile = ALERT_TACTILE(50, BRL_DOT_1 | BRL_DOT_3 | BRL_DOT_4 | BRL_DOT_6),
-    .tune = "n78@100"
+    .tune = "m78@100"
   },
 
   [ALERT_MARK_SET] = {
-    .tune = "n83@20 n81@15 n79@15 n84@25"
+    .tune = "m83@20 m81@15 m79@15 m84@25"
   },
 
   [ALERT_CLIPBOARD_BEGIN] = {
-    .tune = "n74@40 n86@20"
+    .tune = "m74@40 m86@20"
   },
 
   [ALERT_CLIPBOARD_END] = {
-    .tune = "n86@50 n74@30"
+    .tune = "m86@50 m74@30"
   },
 
   [ALERT_NO_CHANGE] = {
     .tactile = ALERT_TACTILE(30, BRL_DOT_2 | BRL_DOT_3 | BRL_DOT_5 | BRL_DOT_6),
-    .tune = "n79@30 r@30 n79@30 r@30 n79@30"
+    .tune = "m79@30 r@30 m79@30 r@30 m79@30"
   },
 
   [ALERT_TOGGLE_ON] = {
     .tactile = ALERT_TACTILE(30, BRL_DOT_1 | BRL_DOT_2 | BRL_DOT_4 | BRL_DOT_5),
-    .tune = "n74@30 r@30 n79@30 r@30 n86@30"
+    .tune = "m74@30 r@30 m79@30 r@30 m86@30"
   },
 
   [ALERT_TOGGLE_OFF] = {
     .tactile = ALERT_TACTILE(30, BRL_DOT_3 | BRL_DOT_7 | BRL_DOT_6 | BRL_DOT_8),
-    .tune = "n86@30 r@30 n79@30 r@30 n74@30"
+    .tune = "m86@30 r@30 m79@30 r@30 m74@30"
   },
 
   [ALERT_CURSOR_LINKED] = {
-    .tune = "n80@7 n79@7 n76@12"
+    .tune = "m80@7 m79@7 m76@12"
   },
 
   [ALERT_CURSOR_UNLINKED] = {
-    .tune = "n78@7 n79@7 n83@20"
+    .tune = "m78@7 m79@7 m83@20"
   },
 
   [ALERT_SCREEN_FROZEN] = {
     .message = strtext("Frozen"),
-    .tune = "n58@5 n59 n60 n61 n62 n63 n64 n65 n66 n67 n68 n69 n70 n71 n72 n73 n74 n76 n78 n80 n83 n86 n90 n95"
+    .tune = "m58@5 m59 m60 m61 m62 m63 m64 m65 m66 m67 m68 m69 m70 m71 m72 m73 m74 m76 m78 m80 m83 m86 m90 m95"
   },
 
   [ALERT_SCREEN_UNFROZEN] = {
     .message = strtext("Unfrozen"),
-    .tune = "n95@5 n90 n86 n83 n80 n78 n76 n74 n73 n72 n71 n70 n69 n68 n67 n66 n65 n64 n63 n62 n61 n60 n59 n58"
+    .tune = "m95@5 m90 m86 m83 m80 m78 m76 m74 m73 m72 m71 m70 m69 m68 m67 m66 m65 m64 m63 m62 m61 m60 m59 m58"
   },
 
   [ALERT_FREEZE_REMINDER] = {
-    .tune = "n60@50 r@30 n60@50"
+    .tune = "m60@50 r@30 m60@50"
   },
 
   [ALERT_WRAP_DOWN] = {
     .tactile = ALERT_TACTILE(20, BRL_DOT_4 | BRL_DOT_5 | BRL_DOT_6 | BRL_DOT_8),
-    .tune = "n86@6 n74@6 n62@6 n50@10"
+    .tune = "m86@6 m74@6 m62@6 m50@10"
   },
 
   [ALERT_WRAP_UP] = {
     .tactile = ALERT_TACTILE(20, BRL_DOT_1 | BRL_DOT_2 | BRL_DOT_3 | BRL_DOT_7),
-    .tune = "n50@6 n62@6 n74@6 n86@10"
+    .tune = "m50@6 m62@6 m74@6 m86@10"
   },
 
   [ALERT_SKIP_FIRST] = {
     .tactile = ALERT_TACTILE(30, BRL_DOT_1 | BRL_DOT_4 | BRL_DOT_7 | BRL_DOT_8),
-    .tune = "r@40 n62@4 n67@6 n74@8 r@25"
+    .tune = "r@40 m62@4 m67@6 m74@8 r@25"
   },
 
   [ALERT_SKIP_ONE] = {
-    .tune = "n74@10 r@18"
+    .tune = "m74@10 r@18"
   },
 
   [ALERT_SKIP_SEVERAL] = {
-    .tune = "n73@20 r@1"
+    .tune = "m73@20 r@1"
   },
 
   [ALERT_BOUNCE] = {
     .tactile = ALERT_TACTILE(50, BRL_DOT_1 | BRL_DOT_2 | BRL_DOT_3 | BRL_DOT_4 | BRL_DOT_5 | BRL_DOT_6 | BRL_DOT_7 | BRL_DOT_8),
-    .tune = "n98@6 n86@6 n74@6 n62@6 n50@10"
+    .tune = "m98@6 m86@6 m74@6 m62@6 m50@10"
   },
 
   [ALERT_ROUTING_STARTED] = {
-    .tune = "n55@10 r@60 n60@15"
+    .tune = "m55@10 r@60 m60@15"
   },
 
   [ALERT_ROUTING_SUCCEEDED] = {
-    .tune = "n64@60 n76@20"
+    .tune = "m64@60 m76@20"
   },
 
   [ALERT_ROUTING_FAILED] = {
-    .tune = "n80@80 n79@90 n78@100 n77@100 r@20 n77@100 r@20 n77@150"
+    .tune = "m80@80 m79@90 m78@100 m77@100 r@20 m77@100 r@20 m77@150"
   },
 
-  [ALERT_MODIFIER_NEXT] = {
-    .tune = "n70@60 n74@60 n77@90"
+  [ALERT_MODIFIER_ONCE] = {
+    .tune = "m70@60 m74@60 m77@90"
   },
 
-  [ALERT_MODIFIER_ON] = {
-    .tune = "n70@60 n74@60 n77@60 n82@90"
+  [ALERT_MODIFIER_LOCK] = {
+    .tune = "m70@60 m74@60 m77@60 m82@90"
   },
 
   [ALERT_MODIFIER_OFF] = {
-    .tune = "n82@60 n77@60 n74@60 n70@90"
+    .tune = "m82@60 m77@60 m74@60 m70@90"
   },
 
   [ALERT_CONSOLE_BELL] = {
     .message = strtext("Console Bell"),
-    .tune = "n78@100"
+    .tune = "m78@100"
   },
 
   [ALERT_KEYS_AUTORELEASED] = {
@@ -174,15 +180,15 @@ static const AlertEntry alertTable[] = {
   },
 
   [ALERT_CONTEXT_DEFAULT] = {
-    .tune = "n76@60 n73@60 n69@60 n66@90"
+    .tune = "m76@60 m73@60 m69@60 m66@90"
   },
 
   [ALERT_CONTEXT_PERSISTENT] = {
-    .tune = "n66@60 n69@60 n73@60 n76@90"
+    .tune = "m66@60 m69@60 m73@60 m76@90"
   },
 
   [ALERT_CONTEXT_TEMPORARY] = {
-    .tune = "n66@60 n69@60 n73@90"
+    .tune = "m66@60 m69@60 m73@90"
   },
 };
 
@@ -262,3 +268,20 @@ alert (AlertIdentifier identifier) {
     }
   }
 }
+
+void
+speakAlertMessage (const char *message) {
+#ifdef ENABLE_SPEECH_SUPPORT
+  sayString(&spk, message, SAY_OPT_MUTE_FIRST);
+#endif /* ENABLE_SPEECH_SUPPORT */
+}
+
+void
+speakAlertText (const wchar_t *text) {
+  char *message = getUtf8FromWchars(text, wcslen(text), NULL);
+
+  if (message) {
+    speakAlertMessage(message);
+    free(message);
+  }
+}
