@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2022 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -42,7 +42,7 @@ typedef HANDLE MonitorEntry;
 #elif defined(HAVE_SYS_POLL_H)
 #define ASYNC_CAN_MONITOR_IO
 
-#include <sys/poll.h>
+#include <poll.h>
 typedef struct pollfd MonitorEntry;
 
 #elif defined(GOT_SELECT)
