@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2018 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -11,7 +11,7 @@
  * Foundation; either version 2.1 of the License, or (at your option) any
  * later version. Please see the file LICENSE-LGPL for details.
  *
- * Web Page: http://brltty.com/
+ * Web Page: http://brltty.app/
  *
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
@@ -177,33 +177,6 @@ checkPeerGroup (PeerCredentials *credentials, const MethodDescriptor_group *grou
   return 0;
 }
 
-#elif defined(SO_PEERCRED)
-#define CAN_CHECK_CREDENTIALS
-
-typedef struct ucred PeerCredentials;
-
-static int
-retrievePeerCredentials (PeerCredentials *credentials, int fd) {
-  socklen_t length = sizeof(*credentials);
-  if (getsockopt(fd, SOL_SOCKET, SO_PEERCRED, credentials, &length) != -1) return 1;
-  logSystemError("getsockopt[SO_PEERCRED]");
-  return 0;
-}
-
-static void
-releasePeerCredentials (PeerCredentials *credentials) {
-}
-
-static int
-checkPeerUser (PeerCredentials *credentials, const MethodDescriptor_user *user) {
-  return user->id == credentials->uid;
-}
-
-static int
-checkPeerGroup (PeerCredentials *credentials, const MethodDescriptor_group *group) {
-  return group->id == credentials->gid;
-}
-
 #elif defined(HAVE_GETPEEREID)
 #define CAN_CHECK_CREDENTIALS
 
@@ -233,6 +206,33 @@ checkPeerGroup (PeerCredentials *credentials, const MethodDescriptor_group *grou
   return group->id == credentials->egid;
 }
 
+#elif defined(SO_PEERCRED)
+#define CAN_CHECK_CREDENTIALS
+
+typedef struct ucred PeerCredentials;
+
+static int
+retrievePeerCredentials (PeerCredentials *credentials, int fd) {
+  socklen_t length = sizeof(*credentials);
+  if (getsockopt(fd, SOL_SOCKET, SO_PEERCRED, credentials, &length) != -1) return 1;
+  logSystemError("getsockopt[SO_PEERCRED]");
+  return 0;
+}
+
+static void
+releasePeerCredentials (PeerCredentials *credentials) {
+}
+
+static int
+checkPeerUser (PeerCredentials *credentials, const MethodDescriptor_user *user) {
+  return user->id == credentials->uid;
+}
+
+static int
+checkPeerGroup (PeerCredentials *credentials, const MethodDescriptor_group *group) {
+  return group->id == credentials->gid;
+}
+
 #else /* peer credentials method */
 #warning peer credentials support not available on this platform
 #endif /* peer credentials method */
@@ -316,7 +316,7 @@ authKeyfile_client (AuthDescriptor *auth, FileDescriptor fd, void *data) {
 static int
 authKeyfile_server (AuthDescriptor *auth, FileDescriptor fd, void *data) {
   MethodDescriptor_keyfile *keyfile = data;
-  logMessage(LOG_DEBUG, "checking key file: %s", keyfile->path);
+  logMessage(LOG_CATEGORY(SERVER_EVENTS), "checking key file: %s", keyfile->path);
   return 1;
 }
 
@@ -439,7 +439,7 @@ authGroup_server (AuthDescriptor *auth, FileDescriptor fd, void *data) {
          checkPeerGroup(&auth->peerCredentials, group);
 }
 
-#ifdef USE_POLKIT
+#ifdef HAVE_POLKIT
 #include <polkit/polkit.h>
 
 typedef struct {
@@ -508,34 +508,50 @@ authPolkit_server (AuthDescriptor *auth, FileDescriptor fd, void *data) {
   socklen_t length = sizeof(cred);
 
   if (getsockopt(fd, SOL_SOCKET, SO_PEERCRED, &cred, &length) != -1) {
-    logMessage(LOG_DEBUG, "attempting to authenticate pid %d via polkit", cred.pid);
+    logMessage(LOG_CATEGORY(SERVER_EVENTS),
+      "attempting to authorize client (pid %d, uid %d) via polkit",
+      cred.pid, cred.uid
+    );
 
-    PolkitSubject *subject = polkit_unix_process_new_for_owner(cred.pid, -1, cred.uid);
-    if (subject) {
-      GError *error_local = NULL;
-
-      PolkitAuthorizationResult *result = polkit_authority_check_authorization_sync(
-        polkit->authority,			/* authority */
-        subject,				/* PolkitSubject for client */
-        "org.a11y.brlapi.write-display",		/* name of polkit action */
-        NULL,					/* details */
-        POLKIT_CHECK_AUTHORIZATION_FLAGS_NONE,	/* disallow interaction */
-        NULL,					/* GCancellable */
-        &error_local				/* returned error */
+    if (cred.uid == -1) {
+      logMessage(LOG_CATEGORY(SERVER_EVENTS),
+        "user not specified in credentials"
       );
-
-      if (result) {
-        int isAuthorized = polkit_authorization_result_get_is_authorized(result);
-        g_object_unref(result);
-
-        logMessage(LOG_DEBUG, "polkit_authority_check_authorization_sync returned %d", isAuthorized);
-        return isAuthorized;
-      } else {
-        logSystemError("polkit_authority_check_authorization_sync");
-        g_error_free(error_local);
-      }
     } else {
-      logSystemError("polkit_unix_process_new_for_owner");
+      PolkitSubject *subject = polkit_unix_process_new_for_owner(cred.pid, 0, cred.uid);
+
+      if (subject) {
+        GError *error_local = NULL;
+
+        PolkitAuthorizationResult *result = polkit_authority_check_authorization_sync(
+          polkit->authority,			/* authority */
+          subject,				/* PolkitSubject for client */
+          "org.a11y.brlapi.write-display",		/* name of polkit action */
+          NULL,					/* details */
+          POLKIT_CHECK_AUTHORIZATION_FLAGS_NONE,	/* disallow interaction */
+          NULL,					/* GCancellable */
+          &error_local				/* returned error */
+        );
+
+        if (result) {
+          int isAuthorized = polkit_authorization_result_get_is_authorized(result);
+          g_object_unref(result);
+
+          logMessage(LOG_CATEGORY(SERVER_EVENTS),
+            "polkit_authority_check_authorization_sync returned %d",
+            isAuthorized
+          );
+
+          return isAuthorized;
+        } else {
+          logMessage(LOG_ERR, "polkit_authority_check_authorization_sync error: %s", error_local->message);
+          g_error_free(error_local);
+        }
+
+        g_object_unref(subject);
+      } else {
+        logSystemError("polkit_unix_process_new_for_owner");
+      }
     }
   } else {
     logSystemError("getsockopt[SO_PEERCRED]");
@@ -543,7 +559,7 @@ authPolkit_server (AuthDescriptor *auth, FileDescriptor fd, void *data) {
 
   return 0;
 }
-#endif /* USE_POLKIT */
+#endif /* HAVE_POLKIT */
 #endif /* CAN_CHECK_CREDENTIALS */
 
 /* general functions */
@@ -571,14 +587,14 @@ static const MethodDefinition methodDefinitions[] = {
     .server = authGroup_server
   },
 
-#ifdef USE_POLKIT
+#ifdef HAVE_POLKIT
   { .name = "polkit",
     .initialize = authPolkit_initialize,
     .release = authPolkit_release,
     .client = NULL,
     .server = authPolkit_server
   },
-#endif /* USE_POLKIT */
+#endif /* HAVE_POLKIT */
 #endif /* CAN_CHECK_CREDENTIALS */
 
   {.name = NULL}
@@ -625,7 +641,7 @@ initializeMethodDescriptor (MethodDescriptor *method, const char *parameter) {
     }
   }
 
-  logMessage(LOG_WARNING, "unknown authentication/authorization method: %.*s", nameLength, name);
+  logMessage(LOG_WARNING, "unknown authorization method: %.*s", nameLength, name);
   return 0;
 }
 
