@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2019 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -38,13 +38,6 @@ getTicksPerWave (BeepFrequency frequency) {
   return frequency? (TICKS_PER_SECOND / frequency): 0;
 }
 
-static void
-enableBeeps (void) {
-  static unsigned char status = 0;
-
-  installKernelModule("pcspkr", &status);
-}
-
 int
 canBeep (void) {
   if (beepDevice == INVALID_FILE_DESCRIPTOR) {
@@ -57,7 +50,7 @@ canBeep (void) {
     }
 
     beepDevice = device;
-    enableBeeps();
+    installSpeakerModule();
   }
 
   return 1;
