@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2022 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -18,6 +18,8 @@
 
 #include "prologue.h"
 
+#include <time.h>
+#include <sys/ioctl.h>
 #include <dev/wscons/wsconsio.h>
 
 #include "log.h"
