@@ -1,7 +1,7 @@
 /*
  * libbrlapi - A library providing access to braille terminals for applications.
  *
- * Copyright (C) 2002-2021 by
+ * Copyright (C) 2002-2023 by
  *   Samuel Thibault <Samuel.Thibault@ens-lyon.org>
  *   Sébastien Hinderer <Sebastien.Hinderer@ens-lyon.org>
  *
@@ -36,6 +36,7 @@
 #include <locale.h>
 #include <assert.h>
 #include <limits.h>
+#include <unistd.h>
 
 #ifndef __MINGW32__
 #ifdef HAVE_LANGINFO_H
@@ -74,7 +75,7 @@
 #include <sys/select.h>
 #endif /* HAVE_SYS_SELECT_H */
 #ifdef HAVE_SYS_POLL_H
-#include <sys/poll.h>
+#include <poll.h>
 #endif /* HAVE_SYS_POLL_H */
 #endif /* __MINGW32__ */
 
@@ -385,7 +386,8 @@ static ssize_t brlapi__doWaitForPacket(brlapi_handle_t *handle, brlapi_packetTyp
   int ret = 0;
 
   struct timeval now;
-  int delay = 0;
+  long delay = 0;
+  int polled = 0; /* Whether we have polled at least once */
 
   do {
     if (deadline) {
@@ -393,10 +395,15 @@ static ssize_t brlapi__doWaitForPacket(brlapi_handle_t *handle, brlapi_packetTyp
       delay = (deadline->tv_sec  - now.tv_sec ) * 1000 +
 	      (deadline->tv_usec - now.tv_usec) / 1000;
       if (delay < 0) {
-	/* The deadline has expired, don't wait more */
-	return -4;
+	if (polled) {
+	  /* The deadline has expired, don't wait more */
+	  return -4;
+	}
+	/* Poll at least once */
+	delay = 0;
       }
     }
+    polled = 1;
 #ifdef __MINGW32__
     DWORD dw;
     dw = WaitForSingleObject(handle->packet.overl.hEvent, deadline ? delay : INFINITE);
@@ -1630,6 +1637,8 @@ static int getControllingTty(void)
   /*if ((env = getenv("WINDOW")) && sscanf(env, "%u", &tty) == 1) return tty;*/
   if ((env = getenv("WINDOWID")) && sscanf(env, "%u", &tty) == 1) return tty;
   if ((env = getenv("CONTROLVT")) && sscanf(env, "%u", &tty) == 1) return tty;
+  if (!getenv("DISPLAY") && !getenv("WAYLAND_DISPLAY"))
+    if ((env = getenv("XDG_VTNR")) && sscanf(env, "%u", &tty) == 1) return tty;
 
 #ifdef WINDOWS
   if (CHECKGETPROC("kernel32.dll", GetConsoleWindow))
@@ -1650,20 +1659,46 @@ static int getControllingTty(void)
 #endif /* WINDOWS */
 
 #ifdef linux
-  {
-    FILE *stream;
-
-    if ((stream = fopen("/proc/self/stat", "r"))) {
-      int vt;
-      int ok = fscanf(stream, "%*d %*s %*c %*d %*d %*d %d", &tty) == 1;
-      fclose(stream);
-
-      if (ok && (major(tty) == TTY_MAJOR)) {
-        vt = minor(tty);
-        if ((vt >= 1) && (vt <= MAXIMUM_VIRTUAL_CONSOLE)) return vt;
-      }
+  do {
+    int fd = open("/proc/self/stat", O_RDONLY, 0);
+    if (fd < 0) {
+      break;
     }
-  }
+
+    char buf[256];
+    ssize_t read_bytes = read(fd, buf, sizeof(buf) - 1);
+    close(fd);
+    if (read_bytes < 0) {
+      break;
+    }
+    /* Append null at the end of buffer to ensure that sscanf doesn't
+     * overflows.*/
+    buf[read_bytes] = '\0';
+
+    /* The line format is "$pid ($comm) $state ...
+     * where comm may contain space and `)` as part of its name, ex:
+     * 12345 (some proc (foo)) R ...
+     * So to process valid identified after comm, we should skip comm by
+     * getting the 1st `)` from end.
+     */
+    char *ptr = buf + read_bytes - 1;
+    while (ptr > buf && *ptr != ')') {
+      ptr -= 1;
+    }
+    if (*ptr != ')') {
+      break;
+    }
+    /* Point to space after ($comm). */
+    ptr += 1;
+
+    int vt;
+    int ok = sscanf(ptr, " %*c %*d %*d %*d %d", &tty) == 1;
+
+    if (ok && (major(tty) == TTY_MAJOR)) {
+      vt = minor(tty);
+      if ((vt >= 1) && (vt <= MAXIMUM_VIRTUAL_CONSOLE)) return vt;
+    }
+  } while (0);
 #endif /* linux */
 
   return -1;
@@ -1709,7 +1744,10 @@ int BRLAPI_STDCALL brlapi__enterTtyModeWithPath(brlapi_handle_t *handle, const i
     return -1;
   }
 
-  if (brlapi__getDisplaySize(handle, &handle->brlx, &handle->brly)<0) return -1;
+  if (brlapi__getDisplaySize(handle, &handle->brlx, &handle->brly)<0){
+    pthread_mutex_unlock(&handle->state_mutex);
+    return -1;
+  }
 
   /* Clear key buffer before taking the tty, just in case... */
   pthread_mutex_lock(&handle->read_mutex);
@@ -1736,12 +1774,12 @@ int BRLAPI_STDCALL brlapi__enterTtyModeWithPath(brlapi_handle_t *handle, const i
       } else {
 	char **sessions;
 	/* Not even logind knows :/ we are probably logged in from gdm */
-	ret = sd_uid_get_sessions(getuid(), 0, &sessions);
+	int nsessions = sd_uid_get_sessions(getuid(), 0, &sessions);
 
-	if (ret > 0) {
+	if (nsessions > 0) {
 	  int i, chosen = -1;
 
-	  for (i = 0; i < ret; i++) {
+	  for (i = 0; i < nsessions; i++) {
 	    char *type;
 
 	    ret = sd_session_get_type(sessions[i], &type);
@@ -1763,7 +1801,7 @@ int BRLAPI_STDCALL brlapi__enterTtyModeWithPath(brlapi_handle_t *handle, const i
 
 	  if (chosen >= 0) sd_session_get_vt(sessions[i], &vtnr);
 
-	  for (i = 0; i < ret; i++) free(sessions[i]);
+	  for (i = 0; i < nsessions; i++) free(sessions[i]);
 	  free(sessions);
 	}
       }
