@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2022 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -24,7 +24,7 @@
 #include <errno.h>
 
 #include "program.h"
-#include "options.h"
+#include "cmdline.h"
 #include "log.h"
 #include "strfmt.h"
 #include "parse.h"
@@ -63,34 +63,6 @@ static char *opt_writeFeature;
 static int opt_echoInput;
 static char *opt_inputTimeout;
 
-static const char *parseBytesHelp[] = {
-  strtext("Bytes may be separated by whitespace."),
-  strtext("Each byte is either two hexadecimal digits or [zero or more braille dot numbers within brackets]."),
-  strtext("A byte may optionally be followed by an asterisk [*] and a decimal count (1 if not specified)."),
-  strtext("The first byte is the report number (00 for no report number)."),
-};
-
-static
-STR_BEGIN_FORMATTER(formatParseBytesHelp, unsigned int index)
-  switch (index) {
-    case 0: {
-      const char *const *sentence = parseBytesHelp;
-      const char *const *end = sentence + ARRAY_COUNT(parseBytesHelp);
-
-      while (sentence < end) {
-        if (STR_LENGTH) STR_PRINTF(" ");
-        STR_PRINTF("%s", gettext(*sentence));
-        sentence += 1;
-      }
-
-      break;
-    }
-
-    default:
-      break;
-  }
-STR_END_FORMATTER
-
 BEGIN_OPTION_TABLE(programOptions)
   { .word = "match-usb-devices",
     .letter = 'u',
@@ -213,18 +185,14 @@ BEGIN_OPTION_TABLE(programOptions)
     .letter = 'w',
     .argument = strtext("bytes"),
     .setting.string = &opt_writeReport,
-    .flags = OPT_Format,
-    .description = strtext("Write (set) an output report. %s"),
-    .strings.format = formatParseBytesHelp
+    .description = strtext("Write (set) an output report (see below)."),
   },
 
   { .word = "write-feature",
     .letter = 'W',
     .argument = strtext("bytes"),
     .setting.string = &opt_writeFeature,
-    .flags = OPT_Format,
-    .description = strtext("Write (set) a feature report. %s"),
-    .strings.format = formatParseBytesHelp
+    .description = strtext("Write (set) a feature report (see below)."),
   },
 
   { .word = "echo-input",
@@ -239,7 +207,15 @@ BEGIN_OPTION_TABLE(programOptions)
     .setting.string = &opt_inputTimeout,
     .description = strtext("The input timeout (in seconds).")
   },
-END_OPTION_TABLE
+END_OPTION_TABLE(programOptions)
+
+static
+BEGIN_USAGE_NOTES(usageNotes)
+  "When writing a report or feature, the bytes don't need to be, but can be, separated from one another by whitespace.",
+  "Each byte is either two hexadecimal digits or zero or more braille dot numbers within [square brackets].",
+  "A byte may optionally be followed by an asterisk [*] and a decimal count - if not specified, 1 is assumed.",
+  "The first byte is the report number - specify 00 for no report number.",
+END_USAGE_NOTES
 
 static FILE *outputStream;
 static int outputError;
@@ -1081,9 +1057,14 @@ performActions (HidDevice *device) {
 int
 main (int argc, char *argv[]) {
   {
-    static const OptionsDescriptor descriptor = {
-      OPTION_TABLE(programOptions),
+    const CommandLineDescriptor descriptor = {
+      .options = &programOptions,
       .applicationName = "brltty-hid",
+
+      .usage = {
+        .purpose = strtext("Find HID devices, list report descriptors, read/write reports/features, or monitor input from a HID device."),
+        .notes = USAGE_NOTES(usageNotes),
+      }
     };
 
     PROCESS_OPTIONS(descriptor, argc, argv);
