@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2022 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -22,7 +22,7 @@
 #include <string.h>
 
 #include "program.h"
-#include "options.h"
+#include "cmdline.h"
 #include "log.h"
 #include "file.h"
 #include "parse.h"
@@ -73,7 +73,6 @@ BEGIN_OPTION_TABLE(programOptions)
 
   { .word = "tables-directory",
     .letter = 'T',
-    .flags = OPT_Hidden,
     .argument = strtext("directory"),
     .setting.string = &opt_tablesDirectory,
     .internal.setting = TABLES_DIRECTORY,
@@ -83,14 +82,13 @@ BEGIN_OPTION_TABLE(programOptions)
 
   { .word = "drivers-directory",
     .letter = 'D',
-    .flags = OPT_Hidden,
     .argument = strtext("directory"),
     .setting.string = &opt_driversDirectory,
     .internal.setting = DRIVERS_DIRECTORY,
     .internal.adjust = fixInstallPath,
     .description = strtext("Path to directory for loading drivers.")
   },
-END_OPTION_TABLE
+END_OPTION_TABLE(programOptions)
 
 static void *driverObject;
 
@@ -284,11 +282,16 @@ main (int argc, char *argv[]) {
   ProgramExitStatus exitStatus = PROG_EXIT_SUCCESS;
 
   {
-    static const OptionsDescriptor descriptor = {
-      OPTION_TABLE(programOptions),
+    const CommandLineDescriptor descriptor = {
+      .options = &programOptions,
       .applicationName = "brltty-ktb",
-      .argumentsSummary = "table-name"
+
+      .usage = {
+        .purpose = strtext("check a key table, list the key naems it can use, or write the key bindings it defines in useful formats."),
+        .parameters = "table-name",
+      }
     };
+
     PROCESS_OPTIONS(descriptor, argc, argv);
   }
 
@@ -384,6 +387,10 @@ void
 alert (AlertIdentifier identifier) {
 }
 
+void
+speakAlertText (const wchar_t *text) {
+}
+
 #include "api_control.h"
 
 const ApiMethods api;
