@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2022 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -135,14 +135,11 @@ extern void getTimeFormattingData (TimeFormattingData *fmt);
 extern STR_DECLARE_FORMATTER(formatBrailleTime, const TimeFormattingData *fmt);
 
 extern int isContracted;
-extern int contractedLength;
-extern int contractedStart;
 extern int contractedTrack;
-extern int *contractedOffsets;
+extern BrailleRowDescriptor *getBrailleRowDescriptor (unsigned int row);
+extern int getCursorOffsetForContracting (void);
 
 extern int isContracting (void);
-extern int getUncontractedCursorOffset (int x, int y);
-extern int getContractedCursor (void);
 extern int getContractedLength (unsigned int outputLimit);
 
 extern ContractionTable *contractionTable;
@@ -153,7 +150,7 @@ extern ProgramExitStatus brlttyPrepare (int argc, char *argv[]);
 extern ProgramExitStatus brlttyStart (void);
 
 extern void setPreferences (const PreferenceSettings *newPreferences);
-extern int loadPreferences (void);
+extern int loadPreferences (int reset);
 extern int savePreferences (void);
 
 extern unsigned char getCursorDots (const unsigned char *setting);
@@ -167,6 +164,7 @@ extern unsigned char getSpeechCursorDots (void);
 extern int setSpeechCursorDots (unsigned char dots);
 
 extern BrailleDisplay brl;			/* braille driver reference */
+extern int haveBrailleDisplay (void);
 
 extern unsigned int textStart;
 extern unsigned int textCount;
@@ -194,19 +192,13 @@ extern void forgetDevices (void);
 extern void reconfigureBrailleWindow (void);
 extern int haveStatusCells (void);
 
-typedef enum {
-  SCT_WORD,
-  SCT_NONWORD,
-  SCT_SPACE
-} ScreenCharacterType;
-
-extern ScreenCharacterType getScreenCharacterType (const ScreenCharacter *character);
 extern int findFirstNonSpaceCharacter (const ScreenCharacter *characters, int count);
 extern int findLastNonSpaceCharacter (const ScreenCharacter *characters, int count);
 extern int isAllSpaceCharacters (const ScreenCharacter *characters, int count);
 
 #ifdef ENABLE_SPEECH_SUPPORT
 extern SpeechSynthesizer spk;
+extern int haveSpeechSynthesizer (void);
 extern int opt_quietIfNoBraille;
 
 extern int isAutospeakActive (void);
