@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2021 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -384,7 +384,16 @@ allocateCharacterClasses (ContractionTableData *ctd) {
 
 const wchar_t *
 getContractionTableOpcodeName (ContractionTableOpcode opcode) {
-  return opcodeNames[opcode];
+  const wchar_t *name = NULL;
+
+  if (opcode >= 0) {
+    if (opcode < ARRAY_COUNT(opcodeNames)) {
+      name = opcodeNames[opcode];
+    }
+  }
+
+  if (!name) name = WS_C("<unknown>");
+  return name;
 }
 
 static ContractionTableOpcode
@@ -600,7 +609,7 @@ static DATA_OPERANDS_PROCESSOR(processContractionTableDirective) {
       }
 
       default:
-        reportDataError(file, "unimplemented opcode: %" PRIws, opcodeNames[opcode]);
+        reportDataError(file, "unimplemented opcode: %" PRIws, getContractionTableOpcodeName(opcode));
         break;
     }
 
@@ -697,18 +706,6 @@ initializeCommonFields (ContractionTable *table) {
   table->rules.array = NULL;
   table->rules.size = 0;
   table->rules.count = 0;
-
-  table->cache.input.characters = NULL;
-  table->cache.input.size = 0;
-  table->cache.input.count = 0;
-
-  table->cache.output.cells = NULL;
-  table->cache.output.size = 0;
-  table->cache.output.count = 0;
-
-  table->cache.offsets.array = NULL;
-  table->cache.offsets.size = 0;
-  table->cache.offsets.count = 0;
 }
 
 static void
@@ -728,21 +725,6 @@ destroyCommonFields (ContractionTable *table) {
     free(table->rules.array);
     table->rules.array = NULL;
   }
-
-  if (table->cache.input.characters) {
-    free(table->cache.input.characters);
-    table->cache.input.characters = NULL;
-  }
-
-  if (table->cache.output.cells) {
-    free(table->cache.output.cells);
-    table->cache.output.cells = NULL;
-  }
-
-  if (table->cache.offsets.array) {
-    free(table->cache.offsets.array);
-    table->cache.offsets.array = NULL;
-  }
 }
 
 static void
