@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2021 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -20,9 +20,6 @@
 
 #include <string.h>
 
-#ifdef HAVE_ICU
-#endif /* HAVE_ICU */
-
 #include "log.h"
 #include "ctb_translate.h"
 #include "ttb.h"
@@ -55,7 +52,7 @@ findLineBreakOpportunities (
   BrailleContractionData *bcd,
   LineBreakOpportunitiesState *lbo,
   unsigned char *opportunities,
-  const wchar_t *characters, unsigned int limit
+  const wchar_t *characters, unsigned int end
 ) {
   /* UAX #14: Line Breaking Properties
    * http://unicode.org/reports/tr14/
@@ -70,7 +67,7 @@ findLineBreakOpportunities (
    * 9  digits
    */
 
-  while (lbo->index <= limit) {
+  while (lbo->index <= end) {
     unsigned char *opportunity = &opportunities[lbo->index];
 
     lbo->previous = lbo->before;
@@ -439,8 +436,6 @@ findLineBreakOpportunities (
 }
 
 #else /* HAVE_ICU */
-#include "ascii.h"
-
 typedef struct {
   unsigned int index;
   int wasSpace;
@@ -457,9 +452,9 @@ findLineBreakOpportunities (
   BrailleContractionData *bcd,
   LineBreakOpportunitiesState *lbo,
   unsigned char *opportunities,
-  const wchar_t *characters, unsigned int limit
+  const wchar_t *characters, unsigned int end
 ) {
-  while (lbo->index <= limit) {
+  while (lbo->index <= end) {
     int isSpace = testCharacter(bcd, characters[lbo->index], CTC_Space);
     opportunities[lbo->index] = lbo->wasSpace && !isSpace;
 
@@ -469,6 +464,19 @@ findLineBreakOpportunities (
 }
 #endif /* HAVE_ICU */
 
+static int
+isLineBreakOpportunity (
+  BrailleContractionData *bcd,
+  LineBreakOpportunitiesState *lbo,
+  unsigned char *opportunities
+) {
+  unsigned int index = getInputConsumed(bcd);
+  if (index == getInputCount(bcd)) return 1;
+
+  findLineBreakOpportunities(bcd, lbo, opportunities, bcd->input.begin, index);
+  return opportunities[index];
+}
+
 static inline ContractionTableHeader *
 getContractionTableHeader (BrailleContractionData *bcd) {
   return bcd->table->data.internal.header.fields;
@@ -501,6 +509,59 @@ getContractionTableCharacter (BrailleContractionData *bcd, wchar_t character) {
   return NULL;
 }
 
+static int
+addRule (BrailleContractionData *bcd, ContractionTableRule *rule) {
+  ContractionTable *table = bcd->table;
+
+  if (table->rules.count == table->rules.size) {
+    size_t newSize = table->rules.size + 10;
+    ContractionTableRule **newArray = realloc(table->rules.array, ARRAY_SIZE(newArray, newSize));
+
+    if (!newArray) {
+      logMallocError();
+      return 0;
+    }
+
+    table->rules.array = newArray;
+    table->rules.size = newSize;
+  }
+
+  table->rules.array[table->rules.count++] = rule;
+  return 1;
+}
+
+static size_t
+makeDecomposedBraille (BrailleContractionData *bcd, wchar_t character, BYTE *cells, size_t size) {
+  wchar_t characters[0X10];
+  size_t characterCount = decomposeCharacter(character, characters, ARRAY_COUNT(characters));
+
+  if (characterCount > 1) {
+    BYTE *from = cells;
+    const BYTE *end = from + size;
+    unsigned int characterIndex = 1;
+
+    while (1) {
+      wchar_t character = characters[characterIndex];
+      const CharacterEntry *entry = getCharacterEntry(bcd, character);
+      if (!entry) break;
+      if (character != entry->value) break;
+
+      const ContractionTableRule *rule = entry->always;
+      if (!rule) break;
+
+      unsigned int cellCount = rule->replen;
+      if (!cellCount) break;
+      if ((end - from) < cellCount) break;
+      from = mempcpy(from, &rule->findrep[rule->findlen], cellCount);
+
+      if (!characterIndex) return from - cells;
+      if (++characterIndex == characterCount) characterIndex = 0;
+    }
+  }
+
+  return 0;
+}
+
 typedef struct {
   BrailleContractionData *bcd;
   CharacterEntry *character;
@@ -509,60 +570,96 @@ typedef struct {
 static int
 setAlwaysRule (wchar_t character, void *data) {
   SetAlwaysRuleData *sar = data;
-  const ContractionTableCharacter *ctc = getContractionTableCharacter(sar->bcd, character);
+  BrailleContractionData *bcd = sar->bcd;
+
+  CharacterEntry *entry = sar->character;
+  const ContractionTableCharacter *ctc = getContractionTableCharacter(bcd, character);
 
   if (ctc) {
     ContractionTableOffset offset = ctc->always;
 
     if (offset) {
-      const ContractionTableRule *rule = getContractionTableItem(sar->bcd, offset);
+      const ContractionTableRule *rule = getContractionTableItem(bcd, offset);
 
       if (rule->replen) {
-        sar->character->always = rule;
+        entry->always = rule;
         return 1;
       }
     }
   }
 
+  if (character == entry->value) {
+    BYTE cells[0X100];
+    size_t count = makeDecomposedBraille(bcd, character, cells, sizeof(cells));
+
+    {
+      unsigned int position;
+      findCharacterEntry(bcd, character, &position);
+
+      entry = &bcd->table->characters.array[position];
+      sar->character = entry;
+    }
+
+    if (count) {
+      ContractionTableRule *rule;
+      size_t size = sizeof(*rule) + sizeof(character) + count;
+
+      if ((rule = malloc(size))) {
+        memset(rule, 0, sizeof(*rule));
+        rule->opcode = CTO_Always;
+
+        rule->findrep[0] = character;
+        memcpy(&rule->findrep[rule->findlen = 1], cells, (rule->replen = count));
+
+        if (addRule(bcd, rule)) {
+          entry->always = rule;
+          return 1;
+        }
+
+        free(rule);
+      }
+    }
+  }
+
   return 0;
 }
 
-static const ContractionTableRule *
-getAlwaysRule (BrailleContractionData *bcd, wchar_t character) {
-  const CharacterEntry *entry = getCharacterEntry(bcd, character);
-
-  return entry? entry->always: NULL;
-}
-
-static wchar_t
-getBestCharacter (BrailleContractionData *bcd, wchar_t character) {
-  const ContractionTableRule *rule = getAlwaysRule(bcd, character);
-
-  return rule? rule->findrep[0]: 0;
-}
-
-static int
-sameCharacters (BrailleContractionData *bcd, wchar_t character1, wchar_t character2) {
-  wchar_t best1 = getBestCharacter(bcd, character1);
-
-  return best1 && (best1 == getBestCharacter(bcd, character2));
-}
-
 static wchar_t
 toLowerCase (BrailleContractionData *bcd, wchar_t character) {
   const CharacterEntry *entry = getCharacterEntry(bcd, character);
   return entry? entry->lowercase: character;
 }
 
-static int
-checkCurrentRule (BrailleContractionData *bcd, const wchar_t *source) {
-  const wchar_t *character = bcd->current.rule->findrep;
-  int count = bcd->current.length;
+static const ContractionTableRule *
+getAlwaysRule (BrailleContractionData *bcd, wchar_t character) {
+  const CharacterEntry *entry = getCharacterEntry(bcd, toLowerCase(bcd, character));
+  return entry? entry->always: NULL;
+}
 
-  while (count) {
-    if (toLowerCase(bcd, *source) != toLowerCase(bcd, *character)) return 0;
-    --count, ++source, ++character;
+static wchar_t
+getBestCharacter (BrailleContractionData *bcd, wchar_t character) {
+  const ContractionTableRule *rule = getAlwaysRule(bcd, character);
+  return rule? rule->findrep[0]: 0;
+}
+
+static int
+sameCharacters (BrailleContractionData *bcd, wchar_t character1, wchar_t character2) {
+  wchar_t best1 = getBestCharacter(bcd, character1);
+  return best1 && (best1 == getBestCharacter(bcd, character2));
+}
+
+static int
+matchCurrentRule (BrailleContractionData *bcd) {
+  const wchar_t *input = bcd->input.current;
+  const wchar_t *find = bcd->current.rule->findrep;
+  const wchar_t *findEnd = find + bcd->current.length;
+
+  while (find < findEnd) {
+    if (toLowerCase(bcd, *input++) != toLowerCase(bcd, *find++)) {
+      return 0;
+    }
   }
+
   return 1;
 }
 
@@ -606,35 +703,54 @@ isEnding (BrailleContractionData *bcd) {
   return 1;
 }
 
+static void
+setCurrentRule (BrailleContractionData *bcd, const ContractionTableRule *rule) {
+  bcd->current.rule = rule;
+  bcd->current.opcode = bcd->current.rule->opcode;
+  bcd->current.length = bcd->current.rule->findlen;
+  setAfter(bcd, bcd->current.length);
+}
+
 static int
 selectRule (BrailleContractionData *bcd, int length) {
+  if (length < 1) return 0;
+
   int ruleOffset;
   int maximumLength;
 
-  if (length < 1) return 0;
   if (length == 1) {
-    const ContractionTableCharacter *ctc = getContractionTableCharacter(bcd, toLowerCase(bcd, *bcd->input.current));
-    if (!ctc) return 0;
+    wchar_t character = toLowerCase(bcd, *bcd->input.current);
+    const ContractionTableCharacter *ctc = getContractionTableCharacter(bcd, character);
+
+    if (!ctc) {
+      const CharacterEntry *entry = getCharacterEntry(bcd, character);
+      if (!entry) return 0;
+
+      const ContractionTableRule *rule = entry->always;
+      if (!rule) return 0;
+
+      setCurrentRule(bcd, rule);
+      return 1;
+    }
+
     ruleOffset = ctc->rules;
     maximumLength = 1;
   } else {
-    wchar_t characters[2];
-    characters[0] = toLowerCase(bcd, bcd->input.current[0]);
-    characters[1] = toLowerCase(bcd, bcd->input.current[1]);
+    const wchar_t characters[] = {
+      toLowerCase(bcd, bcd->input.current[0]),
+      toLowerCase(bcd, bcd->input.current[1]),
+    };
+
     ruleOffset = getContractionTableHeader(bcd)->rules[CTH(characters)];
     maximumLength = 0;
   }
 
   while (ruleOffset) {
-    bcd->current.rule = getContractionTableItem(bcd, ruleOffset);
-    bcd->current.opcode = bcd->current.rule->opcode;
-    bcd->current.length = bcd->current.rule->findlen;
+    setCurrentRule(bcd, getContractionTableItem(bcd, ruleOffset));
 
     if ((length == 1) ||
         ((bcd->current.length <= length) &&
-         checkCurrentRule(bcd, bcd->input.current))) {
-      setAfter(bcd, bcd->current.length);
-
+         matchCurrentRule(bcd))) {
       if (!maximumLength) {
         maximumLength = bcd->current.length;
 
@@ -643,9 +759,8 @@ selectRule (BrailleContractionData *bcd, int length) {
 #define STATE(c) (testCharacter(bcd, (c), CTC_UpperCase)? CS_UpperSingle: testCharacter(bcd, (c), CTC_LowerCase)? CS_Lower: CS_Any)
 
           CapitalizationState current = STATE(bcd->current.before);
-          int i;
 
-          for (i=0; i<bcd->current.length; i+=1) {
+          for (int i=0; i<bcd->current.length; i+=1) {
             wchar_t character = bcd->input.current[i];
             CapitalizationState next = STATE(character);
 
@@ -874,20 +989,19 @@ clearRemainingOffsets (BrailleContractionData *bcd) {
 
 static int
 contractText_native (BrailleContractionData *bcd) {
+  bcd->previous.opcode = CTO_None;
+
   const wchar_t *srcword = NULL;
-  BYTE *destword = NULL;
-
   const wchar_t *srcjoin = NULL;
-  BYTE *destjoin = NULL;
-
-  BYTE *destlast = NULL;
   const wchar_t *literal = NULL;
 
-  unsigned char lineBreakOpportunities[getInputCount(bcd)];
-  LineBreakOpportunitiesState lbo;
+  BYTE *destword = NULL;
+  BYTE *destjoin = NULL;
+  BYTE *destlast = NULL;
 
+  unsigned char lineBreakOpportunities[getInputCount(bcd) + 1];
+  LineBreakOpportunitiesState lbo;
   prepareLineBreakOpportunitiesState(&lbo);
-  bcd->previous.opcode = CTO_None;
 
   while (bcd->input.current < bcd->input.end) {
     int wasLiteral = bcd->input.current == literal;
@@ -936,8 +1050,10 @@ contractText_native (BrailleContractionData *bcd) {
         int outputLength = bcd->output.end - bcd->output.current;
 
         contractText(
-          bcd->table, inputBuffer, &inputLength,
-          bcd->output.current, &outputLength, NULL, CTB_NO_CURSOR
+          bcd->table, NULL,
+          inputBuffer, &inputLength,
+          bcd->output.current, &outputLength,
+          NULL, CTB_NO_CURSOR
         );
 
         bcd->output.current += outputLength;
@@ -948,7 +1064,7 @@ contractText_native (BrailleContractionData *bcd) {
       if (getContractionTableHeader(bcd)->numberSign && (bcd->previous.opcode != CTO_MidNum) &&
           !testBefore(bcd, CTC_Digit) && testCurrent(bcd, CTC_Digit)) {
         if (!putSequence(bcd, getContractionTableHeader(bcd)->numberSign)) break;
-      } else if (getContractionTableHeader(bcd)->englishLetterSign && testCurrent(bcd, CTC_Letter)) {
+      } else if (getContractionTableHeader(bcd)->letterSign && testCurrent(bcd, CTC_Letter)) {
         if ((bcd->current.opcode == CTO_Contraction) ||
             ((bcd->current.opcode != CTO_EndNum) && testBefore(bcd, CTC_Digit)) ||
             (testCurrent(bcd, CTC_Letter) &&
@@ -960,7 +1076,7 @@ contractText_native (BrailleContractionData *bcd) {
               (testNext(bcd, CTC_Punctuation) &&
                !sameCharacters(bcd, bcd->input.current[1], WC_C('.')) &&
                !sameCharacters(bcd, bcd->input.current[1], WC_C('\'')))))) {
-          if (!putSequence(bcd, getContractionTableHeader(bcd)->englishLetterSign)) break;
+          if (!putSequence(bcd, getContractionTableHeader(bcd)->letterSign)) break;
         }
       }
 
@@ -1030,7 +1146,7 @@ contractText_native (BrailleContractionData *bcd) {
             srcbeg = bcd->input.current - bcd->current.length;
             destbeg = destlast;
 
-            while ((bcd->input.current <= srclim) && checkCurrentRule(bcd, bcd->input.current)) {
+            while ((bcd->input.current <= srclim) && matchCurrentRule(bcd)) {
               clearOffset(bcd);
               clearRemainingOffsets(bcd);
             }
@@ -1071,8 +1187,7 @@ contractText_native (BrailleContractionData *bcd) {
       bcd->input.current += 1;
     }
 
-    findLineBreakOpportunities(bcd, &lbo, lineBreakOpportunities, bcd->input.begin, getInputConsumed(bcd));
-    if (lineBreakOpportunities[getInputConsumed(bcd)]) {
+    if (isLineBreakOpportunity(bcd, &lbo, lineBreakOpportunities)) {
       srcjoin = bcd->input.current;
       destjoin = bcd->output.current;
 
