@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2022 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -51,39 +51,64 @@
 #include "program.h"
 
 
+static inline int
+allowBackslashAsPathSeparator (void) {
+#if defined(__MINGW32__) || defined(__MSDOS__)
+  return 1;
+#else /* allow backslash */
+  return 0;
+#endif /* allow backslash */
+}
+
+static int
+isDriveLetter (char character) {
+  return !!strchr("ABCDEFGHIJKLMNOPQRSTUVWXYZ", toupper(character));
+}
+
 int
 isPathSeparator (const char character) {
-  return character == PATH_SEPARATOR_CHARACTER;
+  if (character == PATH_SEPARATOR_CHARACTER) return 1;
+
+  if (allowBackslashAsPathSeparator()) {
+    if (character == '\\') {
+      return 1;
+    }
+  }
+
+  return 0;
 }
 
 int
 isAbsolutePath (const char *path) {
   if (isPathSeparator(path[0])) return 1;
 
-#if defined(__MINGW32__) || defined(__MSDOS__)
-  if (strchr("ABCDEFGHIJKLMNOPQRSTUVWXYZ", toupper(path[0]))) {
-    if (path[1] == ':') {
-      return 1;
+  if (allowBackslashAsPathSeparator()) {
+    if (isDriveLetter(path[0])) {
+      if (path[1] == ':') {
+        if (isPathSeparator(path[2])) {
+          return 1;
+        }
+      }
     }
   }
-#endif /* defined(__MINGW32__) || defined(__MSDOS__) */
 
   return 0;
 }
 
 static size_t
-stripPathDelimiter (const char *path, size_t length) {
+stripPathSeparator (const char *path, size_t length) {
   while (length) {
     if (!isPathSeparator(path[length-1])) break;
     length -= 1;
   }
+
   return length;
 }
 
 char *
 getPathDirectory (const char *path) {
   size_t length = strlen(path);
-  size_t end = stripPathDelimiter(path, length);
+  size_t end = stripPathSeparator(path, length);
 
   if (end) {
     while (--end) {
@@ -93,7 +118,7 @@ getPathDirectory (const char *path) {
     }
 
     if ((length = end)) {
-      if ((end = stripPathDelimiter(path, length))) {
+      if ((end = stripPathSeparator(path, length))) {
         length = end;
       }
     }
@@ -422,14 +447,16 @@ ensureDirectory (const char *path, int worldWritable) {
     logMessage(LOG_ERR, "cannot access directory: %s: %s", path, strerror(errno));
   } else {
     {
-      char *directory = getPathDirectory(path);
-      if (!directory) return 0;
+      char *parent = getPathDirectory(path);
+      if (!parent) return 0;
+      int exists = 0;
 
-      {
-         int exists = ensureDirectory(directory, 0);
-         free(directory);
-         if (!exists) return 0;
+      if (strcmp(parent, path) != 0) {
+        exists = ensureDirectory(parent, 0);
       }
+
+      free(parent);
+      if (!exists) return 0;
     }
 
     if (createDirectory(path, worldWritable)) {
@@ -1056,6 +1083,45 @@ STR_BEGIN_FORMATTER(formatInputError, const char *file, const int *line, const c
   STR_VPRINTF(format, arguments);
 STR_END_FORMATTER
 
+static void
+detachStandardStream (FILE *stream, const char *name, int output) {
+  const char *nullDevice = "/dev/null";
+
+  if (output) {
+    fflush(stream);
+  }
+
+  if (!freopen(nullDevice, (output? "a": "r"), stream)) {
+    if (errno != ENOENT) {
+      char action[0X40];
+      snprintf(action, sizeof(action), "freopen[%s]", name);
+      logSystemError(action);
+    }
+  }
+}
+
+void
+detachStandardInput (void) {
+  detachStandardStream(stdin, "stdin", 0);
+}
+
+void
+detachStandardOutput (void) {
+  detachStandardStream(stdout, "stdout", 1);
+}
+
+void
+detachStandardError (void) {
+  detachStandardStream(stderr, "stderr", 1);
+}
+
+void
+detachStandardStreams (void) {
+  detachStandardInput();
+  detachStandardOutput();
+  detachStandardError();
+}
+
 #ifdef __MINGW32__
 int
 getConsoleSize (size_t *width, size_t *height) {
