@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2022 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -52,7 +52,15 @@ makeBluetoothResourceIdentifier (GioHandle *handle, char *buffer, size_t size) {
 
 static char *
 getBluetoothResourceName (GioHandle *handle, int timeout) {
-  return bthGetNameOfDevice(handle->connection, timeout);
+  const char *name = bthGetNameOfDevice(handle->connection, timeout);
+
+  if (name) {
+    char *copy = strdup(name);
+    if (copy) return copy;
+    logMallocError();
+  }
+
+  return NULL;
 }
 
 static void *
