@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2022 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -29,6 +29,11 @@ getKeyContext (KeyTable *table, unsigned char context) {
   return NULL;
 }
 
+static inline const KeyContext *
+getCurrentKeyContext (KeyTable *table) {
+  return getKeyContext(table, table->context.current);
+}
+
 static inline int
 isTemporaryKeyContext (const KeyTable *table, const KeyContext *ctx) {
   return ((ctx - table->keyContexts.table) > KTB_CTX_DEFAULT) && !ctx->title;
