@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2022 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -94,9 +94,10 @@ typedef struct {
   wchar_t *name;
   wchar_t *title;
 
-  unsigned isSpecial:1;
-  unsigned isDefined:1;
-  unsigned isReferenced:1;
+  unsigned char isSpecial:1;
+  unsigned char isDefined:1;
+  unsigned char isReferenced:1;
+  unsigned char isIsolated:1;
 
   struct {
     KeyBinding *table;
@@ -118,6 +119,16 @@ typedef struct {
   } mappedKeys;
 } KeyContext;
 
+typedef struct {
+  BoundCommand *commands;
+  unsigned int count;
+} CommandMacro;
+
+typedef struct {
+  char **arguments;
+  unsigned int count;
+} HostCommand;
+
 struct KeyTableStruct {
   wchar_t *title;
 
@@ -168,6 +179,18 @@ struct KeyTableStruct {
     int time;
   } autorelease;
 
+  struct {
+    CommandMacro *table;
+    unsigned int size;
+    unsigned int count;
+  } commandMacros;
+
+  struct {
+    HostCommand *table;
+    unsigned int size;
+    unsigned int count;
+  } hostCommands;
+
   struct {
     const char *logLabel;
     const unsigned char *logKeyEventsFlag;
