@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2022 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -388,6 +388,66 @@ validateFloat (float *value, const char *string, const float *minimum, const flo
 }
 #endif /* NO_FLOAT */
 
+#if defined(HAVE_PWD_H) && defined(HAVE_GRP_H)
+#include <pwd.h>
+#include <grp.h>
+
+int
+validateUser (uid_t *value, const char *string, gid_t *group) {
+  {
+    int integer = geteuid();
+    static const int minimum = 0;
+
+    if (validateInteger(&integer, string, &minimum, NULL)) {
+      *value = integer;
+
+      if (group) {
+        struct passwd *user = getpwuid(*value);
+        *group = user? user->pw_gid: 0;
+      }
+
+      return 1;
+    }
+  }
+
+  {
+    struct passwd *user = getpwnam(string);
+
+    if (user) {
+      *value = user->pw_uid;
+      if (group) *group = user->pw_gid;
+      return 1;
+    }
+  }
+
+  return 0;
+}
+
+int
+validateGroup (gid_t *value, const char *string) {
+  {
+    int integer = geteuid();
+    static const int minimum = 0;
+
+    if (validateInteger(&integer, string, &minimum, NULL)) {
+      *value = integer;
+      return 1;
+    }
+  }
+
+  {
+    struct group *group = getgrnam(string);
+
+    if (group) {
+      *value = group->gr_gid;
+      return 1;
+    }
+  }
+
+  return 0;
+}
+#endif /* defined(HAVE_PWD_H) && defined(HAVE_GRP_H) */
+
 int
 hasQualifier (const char **identifier, const char *qualifier) {
   const char *delimiter = strchr(*identifier, PARAMETER_QUALIFIER_CHARACTER);
