@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2021 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -80,10 +80,11 @@ PREFERENCE_STRING_TABLE(autoreleaseTime,
 )
 
 PREFERENCE_STRING_TABLE(cursorStyle,
-  [csUnderline] = "underline",
-  [csBlock] = "block",
+  [csBottomDots] = "underline",
+  [csAllDots] = "block",
   [csLowerLeftDot] = "dot7",
   [csLowerRightDot] = "dot8",
+  [csNoDots] = "hide",
 )
 
 PREFERENCE_STRING_TABLE(brailleFirmness,
@@ -479,6 +480,18 @@ const PreferenceDefinition preferenceDefinitionTable[] = {
     .setting = &prefs.keyboardLedAlerts
   },
 
+  { .name = "speak-key-context",
+    .defaultValue = DEFAULT_SPEAK_KEY_CONTEXT,
+    .settingNames = &preferenceStringTable_boolean,
+    .setting = &prefs.speakKeyContext
+  },
+
+  { .name = "speak-modifier-key",
+    .defaultValue = DEFAULT_SPEAK_MODIFIER_KEY,
+    .settingNames = &preferenceStringTable_boolean,
+    .setting = &prefs.speakModifierKey
+  },
+
   { .name = "alert-tunes",
     .defaultValue = DEFAULT_ALERT_TUNES,
     .settingNames = &preferenceStringTable_boolean,
