@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2022 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -22,7 +22,7 @@
 #include "service.h"
 
 int
-installService (const char *name, const char *description) {
+installService (const char *name, const char *description, const char *configurationFile) {
   logUnsupportedFeature("service installation");
   return 0;
 }
