@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2022 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -953,9 +953,13 @@ installUinputModule (void) {
 }
 
 static int
-openDevice (const char *path, mode_t flags, int allowModeSubset) {
+openDevice (const char *path, int flags, int allowModeSubset) {
   int descriptor;
 
+  #ifdef O_CLOEXEC
+  flags |= O_CLOEXEC;
+  #endif /* O_CLOEXEC */
+
   if ((descriptor = open(path, flags)) != -1) goto opened;
   if (!allowModeSubset) goto failed;
   if ((flags & O_ACCMODE) != O_RDWR) goto failed;
