@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2022 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -26,7 +26,7 @@
 #endif /* HAVE_(GET|SET)TIMEOFDAY */
 
 #ifdef HAVE_SYS_POLL_H
-#include <sys/poll.h>
+#include <poll.h>
 #endif /* HAVE_SYS_POLL_H */
 
 #ifdef HAVE_SELECT
@@ -73,7 +73,7 @@ getCurrentTime (TimeValue *now) {
     now->nanoseconds = (milliseconds % MSECS_PER_SEC) * NSECS_PER_MSEC;
   }
 
-#elif defined(HAVE_CLOCK_GETTIME) && defined(CLOCK_REALTIME)
+#elif defined(HAVE_CLOCK_GETTIME) && defined(CLOCK_REALTIME) && !defined(__MINGW32__)
   struct timespec ts;
 
   if (clock_gettime(CLOCK_REALTIME, &ts) != -1) {
@@ -86,7 +86,12 @@ getCurrentTime (TimeValue *now) {
 #elif defined(HAVE_GETTIMEOFDAY)
   struct timeval tv;
 
-  if (gettimeofday(&tv, NULL) != -1) {
+  #pragma GCC diagnostic push
+  #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+  int result = gettimeofday(&tv, NULL);
+  #pragma GCC diagnostic pop
+
+  if (result != -1) {
     now->seconds = tv.tv_sec;
     now->nanoseconds = tv.tv_usec * NSECS_PER_USEC;
   } else {
