@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2022 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -154,8 +154,8 @@ getCharacterWidth (wchar_t character) {
   if (category == U_UNASSIGNED) return -1;
   return 1;
 #else /* character width */
-  if (character == NUL) return 0;
-  if (character == DEL) return -1;
+  if (character == ASCII_NUL) return 0;
+  if (character == ASCII_DEL) return -1;
   if (!(character & 0X60)) return -1;
   return 1;
 #endif /* character width */
@@ -202,7 +202,7 @@ getReplacementCharacter (void) {
 #ifdef HAVE_WCHAR_H
  return UNICODE_REPLACEMENT_CHARACTER;
 #else /* HAVE_WCHAR_H */
- return SUB;
+ return ASCII_SUB;
 #endif /* HAVE_WCHAR_H */
 }
 
