@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2021 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -23,10 +23,20 @@
 
 #include "log.h"
 #include "program.h"
-#include "io_usb.h"
 #include "usb_internal.h"
 #include "usb_serial.h"
 #include "usb_adapters.h"
+#include "bitfield.h"
+
+static void
+usbLogSerialProblem (UsbDevice *device, const char *problem) {
+  logMessage(LOG_CATEGORY(SERIAL_IO),
+    "%s: Vendor:%04X Product:%04X",
+    problem,
+    getLittleEndian16(device->descriptor.idVendor),
+    getLittleEndian16(device->descriptor.idProduct)
+  );
+}
 
 int
 usbSkipInitialBytes (UsbInputFilterData *data, unsigned int count) {
@@ -137,6 +147,7 @@ usbSetSerialOperations (UsbDevice *device) {
     }
 
     if (uso) {
+      logMessage(LOG_CATEGORY(SERIAL_IO), "USB adapter: %s", uso->name);
       UsbSerialData *usd = NULL;
 
       if (uso->makeData) {
@@ -158,30 +169,52 @@ usbGetSerialOperations (UsbDevice *device) {
   return device->serial.operations;
 }
 
+UsbSerialData *
+usbGetSerialData (UsbDevice *device) {
+  return device->serial.data;
+}
+
 int
 usbSetSerialParameters (UsbDevice *device, const SerialParameters *parameters) {
+  int ok = 0;
   const UsbSerialOperations *serial = usbGetSerialOperations(device);
 
   if (!serial) {
-    logMessage(LOG_CATEGORY(USB_IO), "no serial operations: vendor=%04X product=%04X",
-               getLittleEndian16(device->descriptor.idVendor),
-               getLittleEndian16(device->descriptor.idProduct));
+    usbLogSerialProblem(device, "no serial operations");
     errno = ENOSYS;
-    return 0;
-  }
-
-  if (serial->setLineConfiguration) {
-    if (!serial->setLineConfiguration(device, parameters->baud, parameters->dataBits, parameters->stopBits, parameters->parity, parameters->flowControl)) return 0;
+  } else if (serial->setLineConfiguration) {
+    ok = serial->setLineConfiguration(
+      device, parameters->baud,
+      parameters->dataBits, parameters->stopBits,
+      parameters->parity, parameters->flowControl
+    );
+  } else if (serial->setLineProperties) {
+    ok = serial->setLineProperties(
+      device, parameters->baud,
+      parameters->dataBits, parameters->stopBits,
+      parameters->parity
+    );
   } else {
-    if (serial->setLineProperties) {
-      if (!serial->setLineProperties(device, parameters->baud, parameters->dataBits, parameters->stopBits, parameters->parity)) return 0;
-    } else {
-      if (!serial->setBaud(device, parameters->baud)) return 0;
-      if (!serial->setDataFormat(device, parameters->dataBits, parameters->stopBits, parameters->parity)) return 0;
+    ok = 1;
+
+    if (!serial->setBaud) {
+      usbLogSerialProblem(device, "setting baud is not supported");
+    } else if (!serial->setBaud(device, parameters->baud)) {
+      ok = 0;
     }
 
-    if (!serial->setFlowControl(device, parameters->flowControl)) return 0;
+    if (!serial->setDataFormat) {
+      usbLogSerialProblem(device, "setting data format is not supported");
+    } else if (!serial->setDataFormat(device, parameters->dataBits, parameters->stopBits, parameters->parity)) {
+      ok = 0;
+    }
+
+    if (!serial->setFlowControl) {
+      usbLogSerialProblem(device, "setting flow control is not supported");
+    } else if (!serial->setFlowControl(device, parameters->flowControl)) {
+      ok = 0;
+    }
   }
 
-  return 1;
+  return ok;
 }
