@@ -1,7 +1,7 @@
 /*
  * XBrlAPI - A background process tinkering with X for proper BrlAPI behavior
  *
- * Copyright (C) 2003-2022 by Samuel Thibault <Samuel.Thibault@ens-lyon.org>
+ * Copyright (C) 2003-2023 by Samuel Thibault <Samuel.Thibault@ens-lyon.org>
  *
  * XBrlAPI comes with ABSOLUTELY NO WARRANTY.
  *
@@ -59,7 +59,7 @@
 #define BRLAPI_NO_DEPRECATED
 #include "brlapi.h"
 
-#include "options.h"
+#include "cmdline.h"
 
 #define debugf(fmt, ...) do { if (verbose) fprintf(stderr, fmt, ## __VA_ARGS__); } while (0)
 
@@ -101,12 +101,6 @@ BEGIN_OPTION_TABLE(programOptions)
     .description = strtext("X display to connect to")
   },
 
-  { .word = "no-daemon",
-    .letter = 'n',
-    .setting.flag = &no_daemon,
-    .description = strtext("Remain a foreground process")
-  },
-
   { .word = "quiet",
     .letter = 'q',
     .setting.flag = &quiet,
@@ -118,7 +112,13 @@ BEGIN_OPTION_TABLE(programOptions)
     .setting.flag = &verbose,
     .description = strtext("Write debugging output to stdout")
   },
-END_OPTION_TABLE
+
+  { .word = "no-daemon",
+    .letter = 'n',
+    .setting.flag = &no_daemon,
+    .description = strtext("Remain a foreground process")
+  },
+END_OPTION_TABLE(programOptions)
 
 /******************************************************************************
  * error handling
@@ -1035,10 +1035,15 @@ static void term_handler(int foo) {
 int
 main (int argc, char *argv[]) {
   {
-    static const OptionsDescriptor descriptor = {
-      OPTION_TABLE(programOptions),
-      .applicationName = "xbrlapi"
+    const CommandLineDescriptor descriptor = {
+      .options = &programOptions,
+      .applicationName = "xbrlapi",
+
+      .usage = {
+        .purpose = strtext("Augment an X session by supporting input typed on the braille device, showing the title of the focused window on the braille display, and switching braille focus to it."),
+      }
     };
+
     PROCESS_OPTIONS(descriptor, argc, argv);
   }
 
