@@ -2,7 +2,7 @@
 # BRLTTY - A background process providing access to the console screen (when in
 #          text mode) for a blind person using a refreshable braille display.
 #
-# Copyright (C) 1995-2022 by The BRLTTY Developers.
+# Copyright (C) 1995-2023 by The BRLTTY Developers.
 #
 # BRLTTY comes with ABSOLUTELY NO WARRANTY.
 #
@@ -632,7 +632,7 @@ proc formatCommandOptionsUsage {options} {
 
 proc showCommandUsage {name optionsDescriptor argumentsUsage getArgumentsUsageSummary} {
    set optionsUsage [formatCommandOptionsUsage $optionsDescriptor]
-   set usage "Usage: $name"
+   set usage "Syntax: $name"
 
    if {[string length $optionsUsage] > 0} {
       append usage " \[-option ...\]"
